/*
    File:            consol_awf1c.cpp
    Author:          Tomas Krejci, 12/09/2008, revised 23/08/2012
    Purpose:         material model for saturated-nonsaturated one-phase flow in deforming medium
    sources:         Lewis and Schrefler's book pp. 86-89, initial material parameters are set for benchmark on page n. 168;
    another sources: geomkp.pdf from michal Sejnoha;
                     
                     W. R. Gardner, Some steady-state solutions of the unsaturated moisture flow equation 
                     to evaporation from a water table, Soil Science 85 (1958), no. 4, 228-232;
                     
                     D. M. Potts and L. Zdravkovic, Finite element analysis in geotechnical engineering theory, 
		     Thomas Telford, London, 1999;

		     M. Th. van Genuchten, A closed equation for predicting the hydraulic conductivity of
		     unsaturated soils, Journal Soil Science Society of America 44 (1980), 892-898;
		     
		     P. Kuklik, J. Mares and M. Sejnoha (1999) The structural strenght of soil
		     from the isotropic consolidation point of view. Computational Mechanics for the Next
		     Millenium. Wang, Lee, Ang (ed.), Vol. 2, Elsevier, Singpore, 797-802;

		     P. Kuklik (2010) Preconsolidation, Structural Strength of soil, and its effect on subsoil 
		     upper structure intraction; Professor thesis

    unknowns:        number of unknowns=1, pw = liqud water pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "constrel.h"
#include "onemediumc.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "consol_awf1c.h"
#include "intpoints.h"


con_awf1matc::con_awf1matc()
{
  //
  emod = 1.3e6;//Young's modulus [Pa]
  //
  nu = 0.4;//Poisson's ratio [-]
  //
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 2.167e6;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  //ka = 0.1e6;//bulk modulus of air [Pa]
  //
  phi0 = 0.297;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhos = 2000.0;//solid grain density [kg/m^3]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  //
  //mua = 1.8e-5;//air viscosity [Pa.s]
  /////////////////////////////////////////////////////////
  ssat = 1.0; //
  //
  sirr = 0.33; //
  //
  //phi0 = 0.45; //
  //
  alfa = 0.1; //
  //
  ksat = 0.1; //
  //
  gamaw = 10000.0; //
  /////////////////////////////////////////////////////////
  hp_min = 0.0; //
  htz = 0.14; //
  r = 100.0; //  
  /////////////////////////////////////////////////////////
  delta = 20.0; //
  expn = 2.0; //
  /////////////////////////////////////////////////////////
  k0 = 32e-11;//
  e0 = 0.52;//
  lam0 = 0.001;//
  lam = 0.03;//
  m = 9;//
  sigma_m_bar = 100000.0;//initial effective mean stress
  /////////////////////////////////////////////////////////
  kappa = 0.012;//
  sigma_m_eff_bar = 28450.0;//initial effective mean stress = preonsolidation pressure
  /////////////////////////////////////////////////////////
}

con_awf1matc::~con_awf1matc()
{}


/**
   function reads parameters
   
   @param in - input file

   12/9/2008, TKr
*/
void con_awf1matc::read(XFILE *in)
{
  
  xfscanf (in,"%ld",&model_type);
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &emod, &nu, &rhos, &alpha, &ks, 
	     &phi0, &kw, &rhow, &muw0, &kintr);
    break;
  }
  case 2:{//fully saturated isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &lam0, &lam, &m, &gamaw, &sigma_m_bar);
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    xfscanf (in,"%lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &alfa, &ksat, &gamaw);
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &hp_min, &htz, &r, &ksat, &gamaw);
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &ssat, &sirr, &phi0, &expn, &ksat, &delta, &gamaw);
    break;
  }
  case 6:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_bar);
    break;
  }
  case 7:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_bar);
    break;
  }
  case 8:{//fully saturated isotropic consolidation according to Kuklik, with changing porosity from MEFEL cam-clay model
    xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf", &k0, &e0, &kappa, &lam, &m, &gamaw, &sigma_m_eff_bar);
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
}




/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  f = get_fw1(pw);
  g = Tp->gr1;//vector of gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g;
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  g = new double [2];
  f = get_fw1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void con_awf1matc::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  g = new double [3];
  f = get_fw1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  g[2] = Tp->gr3;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
  d[2][0] = f*g[2];
}




/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure

   @retval sw - degree of saturation

   12/9/2008, TKr
*/
double con_awf1matc::get_sw(double pw)
{
  double sw,kr;
  double hp,theta,expm;
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    sw = 1.0 - (1.9722e-11)*pow(-pw,2.4279);//p. 167??!!
    break;
  }
  case 2:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    sw = 1.0;
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    kr = get_krw(pw);
    sw = sirr + (ssat - sirr)*kr;//Warick
    //sw = sirr + (ssat - sirr)*pow(kr,1.0/3.0);//Fatt and Klikof
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    kr = get_krw(pw);
    sw = sirr + (ssat - sirr)*kr;//Warick
    //sw = sirr + (ssat - sirr)*pow(kr,1.0/3.0);//Fatt and Klikof
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    hp = pw/gamaw;
    expm = 1.0 - 1.0/expn;
    theta = pow(1+(pow(delta*fabs(hp),expn)),-expm);
    sw = sirr + (ssat - sirr)*theta;
    break;
  }
  case 6:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    sw = 1.0;
    break;
  }
  case 7:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    sw = 1.0;
    break;
  }
  case 8:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    sw = 1.0;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(sw);
}


/**
   function computes partial derivative of degree of saturation with respect to pw, specific wter content
   @param pw - water pressure

   @retval dsw_dpw - partial derivative of degree of saturation with respect to pw

   12/9/2008, TKr
*/
double con_awf1matc::get_dsw_dpw(double pw)
{
  double dsw_dpw;
  double dkr_dpw;
  double hp,expm;

  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    dsw_dpw = 2.4279*(1.9722e-11)*pow(-pw,(2.4279-1.0));//p. 167
    break;
  }
  case 2:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    dkr_dpw = exp(alfa*pw/gamaw)*alfa/gamaw;
    dsw_dpw = (ssat - sirr)*dkr_dpw;//Warick
    //dsw_dpw = (ssat - sirr)*alfa/3.0*pow(dkr_dpw,-2.0/3.0);//Fatt and Klikof
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    hp = pw/gamaw;
    dkr_dpw = pow(10,(hp - hp_min)*log10(r)/htz)*log(10)*log10(r)/gamaw/htz;
    dsw_dpw = (ssat - sirr)*dkr_dpw;//Warick
    //dsw_dpw = (ssat - sirr)*alfa/3.0*pow(dkr_dpw,-2.0/3.0);//Fatt and Klikof
    break;
  }
  case 5:{//partially saturated medium = Van Genuchten model
    hp = pw/gamaw;
    expm = 1.0 - 1.0/expn;
    dsw_dpw = (expm*expn*delta*(ssat - sirr)*pow(delta*fabs(hp),(expn-1)))/
      (pow(1+(pow(delta*fabs(hp),expn)),(expm+1)))/gamaw;
    break;
  }
  case 6:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case 7:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case 8:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  
  return(dsw_dpw);
}


/**
   function computes water relative permeability
   @param pw - water pressure
   
   @retval krw - water relative permeability

   12/9/2008, TKr
*/
double con_awf1matc::get_krw(double pw)
{
  double krw,s;
  double hp,expm;

  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    s = get_sw(pw);
    krw = 1.0+2207*pow((1.0-s),1.0121); //p. 167
    break;
  }
  case 2:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    krw = exp(0.1*pw/gamaw);
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    hp = pw/gamaw;
    krw = pow(10,(hp - hp_min)*log10(r)/htz);
    break;
  }
  case 5:{//partially saturated medium = Van Genuchten model
    expm = 1.0 - 1.0/expn;
    hp = pw/gamaw;
    krw = pow(1.0-pow(delta*fabs(hp),expn)*pow(1.0+pow(delta*fabs(hp),expn),-expm),2.0)/
      pow(1.0+pow(delta*fabs(hp),expn),(expm/2.0));
    //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(3.0/expm)));//Fatt and klikof
    //krw = 1.0/(pow(1.0+(pow(delta*fabs(hp),expn)),(1.0/expm)));//Warrick
    break;
  }
  case 6:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case 7:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    break;
  }
  case 8:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(krw);
}


/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double con_awf1matc::get_phi()
{
  double phi;

  phi = phi0;

  return(phi);
}


/**
   function computes intrinsic permeability

   @retval kintr - intrinsic permeability

   12/9/2008, TKr
*/
double con_awf1matc::get_kintr()
{
  return(kintr);
}


/**
   function returns volume density of soil skeleton

   @retval rhos - volume density of concrete skeleton

   12/9/2008, TKr
*/
double con_awf1matc::get_rhos()
{
  return(rhos);
}


/**
   function returns Biot's constant

   @retval alpha - Biot's constant

   12/9/2008, TKr
*/
double con_awf1matc::get_alpha()
{
  return(alpha);
}


/**
   function returns emod Young's modulus

   @retval emod = Young's modulus

   12/9/2008, TKr
*/
double con_awf1matc::get_emod()
{
  return(emod);
}

/**
   function returns nu = Poisson's ratio

   @retval nu - Poisson's ratio

   12/9/2008, TKr
*/
double con_awf1matc::get_nu()
{
  return(nu);
}


/**
   function returns ks bulk modulus of solid phase (grains) [Pa]

   @retval ks - bulk modulus of solid phase (grains) [Pa]

   12/9/2008, TKr
*/
double con_awf1matc::get_ks()
{
  return(ks);
}


/**
   function returns kw bulk modulus of water [Pa]

   @retval kw - bulk modulus of water [Pa]

   12/9/2008, TKr
*/
double con_awf1matc::get_kw()
{
  return(kw);
}


/**
   function returns muw water viscosity [Pa.s]

   @retval muw - water viscosity [Pa.s]

   12/9/2008, TKr
*/
double con_awf1matc::get_muw()
{
  return(muw0);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kuw - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_kuw(double pw)
{
  double s,kuw;
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book  
    
    s = get_sw(pw);
    
    kuw = -alpha*s;//p. 88
    break;
  }
  case 2:{//fully saturated isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity 
          //from cam-clay model
    kuw = -1.0;
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    break;
  }
  case 6:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 7:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 8:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 

  return(kuw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kwu - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_kwu(double pw)
{
  double kwu;

  kwu = 0.0;

  return(kwu);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kww - conductivity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_kww(double pw, long ipp)
{
  double krw,kww;
  double ks;
  double e,eps_v,sigma_m_eff,e0_bar;

  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    krw = get_krw(pw);
    kww = krw*kintr/muw0;//p. 89
    break;
  }
  case 2:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    //NCL with constant mean effective stress
    eps_v = -lam/(1+e0)*log(-sigma_m_bar);
    e = e0*(1 + eps_v);
    
    kww = -k0*(1+e0)/lam/gamaw*sigma_m_bar*pow((e/e0),m);
    kww = -k0*(1+e0)/lam/gamaw*sigma_m_bar;//debug??!!
    break;
  }
    
  case 3:{//partially saturated medium = Exponential model, Gardner 
    ks  = ksat/86400.0;
    krw = get_krw(pw);
    kww = krw*ks/gamaw;
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    ks  = ksat/86400.0;
    krw = get_krw(pw);
    kww = krw*ks/gamaw;
    break;
  }
  case 5:{//partially saturated medium = Van Genuchten model
    ks  = ksat/86400.0;
    krw = get_krw(pw);
    kww = krw*ks/gamaw;    
    break;
  }
  case 6:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
   
    sigma_m_eff = pw;
    e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);

    if(sigma_m_eff >= sigma_m_eff_bar){//swelling line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      e = e0_bar*(1 + eps_v);
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      e = e0*(1 + eps_v);
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
    }

    break;
  }
  case 7:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    //NCL with changing mean effective stress
    sigma_m_eff = pw;
    eps_v = -lam/(1+e0)*log(-sigma_m_eff);
    e = e0*(1 + eps_v);
    
    kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
    break;
  }
  case 8:{
    //isotropic consolidation according to Kuklik with changing porosity from cam-clay model; 
    //parameters are taken from MEFEL
    

    if(Tm->nontransq != NULL){
      sigma_m_eff_bar = Tm->nontransq[ipp][5]; //actual precosolidation pressure
      sigma_m_eff = Tm->nontransq[ipp][6]; //actual mean effective stress
    }
    
    sigma_m_eff = pw;
    e0_bar = (lam + kappa)*log(-sigma_m_eff_bar);

    if(sigma_m_eff >= sigma_m_eff_bar){//swelling line
      eps_v = -kappa/(1+e0_bar)*log(-sigma_m_eff);
      e = e0_bar*(1 + eps_v);
      kww = -k0*(1+e0_bar)/kappa/gamaw*sigma_m_eff*pow((e/e0),m);
    }
    else{ //NCL
      eps_v = -lam/(1+e0)*log(-sigma_m_eff);
      e = e0*(1 + eps_v);
      kww = -k0*(1+e0)/lam/gamaw*sigma_m_eff*pow((e/e0),m);
    }

    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(kww);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capuw - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_capuw(double pw)
{
  double capuw;
  
  capuw = 0.0;

  return(capuw);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capwu - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_capwu(double pw)
{
  double s,capwu;
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book  
    s = get_sw(pw); 
    capwu = alpha*s;//p. 88
    break;
  }
  case 2:{//fully saturated isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity 
          //from cam-clay model
    capwu = 1.0;
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    break;
  }
  case 6:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 7:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 8:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(capwu);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capww - capacity coefficient

   12/9/2008, TKr
*/
double con_awf1matc::get_capww(double pw)
{
  double sw,dsw_dpw,n,capww;  
  
  dsw_dpw = get_dsw_dpw(pw);
  n = phi0;
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book  
    sw = get_sw(pw);
    //capww = (alpha-n)/ks*sw*(sw + dsw_dpw/n*pw)+n*sw/kw + dsw_dpw;//p. 89 this equation is wrong
    capww = (alpha-n)/ks*sw*(sw + dsw_dpw*pw)+n*sw/kw + n*dsw_dpw;//this equation is correct (different derivation)
    //capww = (alpha-n)/ks+n*sw/kw;//fully saturation state only for debug
    break;
  }
  case 2:{//fully saturated isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity 
          //from cam-clay model
    capww = 1.0; 
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    capww = phi0*dsw_dpw;
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    capww = phi0*dsw_dpw;
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    capww = phi0*dsw_dpw;
    break;
  }
  case 6:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    capww = 1.0; 
    break;
  }
  case 7:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    capww = 1.0; 
    break;
  }
  case 8:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    capww = 1.0; 
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(capww);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure

   @retval fw - first part for right-hand side for continutiy equation

   12/9/2008, TKr
*/
double con_awf1matc::get_fw1(double pw)
{
  double krw,fw1;
  
  krw = get_krw(pw);

  fw1 = krw*kintr/muw0*rhow;//p. 89

  //printf("\n pw = %e, fw1 = %e",pw,fw1);

  //for variation of permeability from cam-clay model
  // lambda = slope of the NCL
  // v_lam = specific volume according to lambda
  // sigma_meff = effective mean stress
  //fw1 = fw1*sigma_meff/lambda*v_lam;  

  return(fw1);

}



/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fu1 - first part for right-hand side for balance equation

   12/9/2008, TKr
*/
double con_awf1matc::get_fu1(double pw)
{
  double sw,n,fu1;  
  
  sw = get_sw(pw);
  n = phi0;
  
  //fu1 = rhos*(1-n) + sw*n*rhow;//p. 88
  fu1 = 0.0;//debug

  return(fu1);

}

/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.

  @param ipp - integration point number in the mechmat ip array.
  @param ido - index of internal variables for given material in the ipp other array

   24/08/2012, TKr
*/
void con_awf1matc::updateval (long ipp,long ido)
{
  long i;
  double pw,s;

  pw = Cmu->ip[ipp].av[0];// *scale_pc;//scaling
  s = get_sw(pw);

  if(Cmu->nonmechtransq != NULL){
    Cmu->nonmechtransq[ipp][0] = alpha*s*pw; //part for effective stress computing
    //Cmu->nonmechtransq[ipp][1] = 0.0;//it is computing in function get_eigstr(pw);
    //Cmu->nonmechtransq[ipp][2] = 0.0;//it is computing in function get_eigstr(pw);
    Cmu->nonmechtransq[ipp][3] = get_sw(pw);//initial value degree of saturation
    Cmu->nonmechtransq[ipp][4] = get_suction(pw);//initial value of suction
  }
}


/**
   This function initializes material model data
   with respect of consistency parameter gamma.

   @param ipp - integration point number
   @param ido - index of internal variables for given material in the ipp other array

   24/08/2012, TKr
*/
void con_awf1matc::initval(long ipp, long ido)
{
  long i;
  double pw,s;

  pw = Cmu->ip[ipp].av[0];// *scale_pc;//scaling
  s = get_sw(pw);
  
  if(Cmu->nonmechtransq != NULL){
    Cmu->nonmechtransq[ipp][0] = alpha*s*pw; //part for effective stress computing
    Cmu->nonmechtransq[ipp][1] = 0.0;//initial value
    Cmu->nonmechtransq[ipp][2] = 0.0;//initial value of total volume strain
    Cmu->nonmechtransq[ipp][3] = get_sw(pw);//initial value degree of saturation
    Cmu->nonmechtransq[ipp][4] = get_suction(pw);//initial value of suction
  }
}


/**
   function computes nonmechanical strains
   
   @retval eigstr - eigenstrain for mechanical part
*/
double con_awf1matc::get_eigstr(double pw)
{
  double s,eigstr;

  switch (model_type){
  case 1:{//Lewis and Schrefler's book  
    s = get_sw(pw);
    eigstr = -alpha*s/emod;
    break;
  }
  case 2:{//fully saturated isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity 
          //from cam-clay model
    eigstr = 1.0/emod;
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    break;
  }
  case 6:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 7:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from cam-clay model
    break;
  }
  case 8:{//fully saturated isotropic consolidation according to Kuklik with changing porosity 
          //from MEFEL cam-clay model
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(eigstr);
}


/**
   function computes suction stress (s = pg - pw = pc);
   @param pw - water pressure

   @retval suction - suction stress [Pa]

   25/9/2012, TKr
*/
double con_awf1matc::get_suction(double pw)
{
  double suction;
  
  switch (model_type){
  case 1:{//Lewis and Schrefler's book
    suction = -pw;
    break;
  }
  case 2:{//isotropic consolidation according to Kuklik, Mares, Sejnoha with changing porosity from cam-clay model
    suction = -pw;
    break;
  }
  case 3:{//partially saturated medium = Exponential model, Gardner 
    suction = -pw;
    break;
  }
  case 4:{//partially saturated medium = Log linear model, Potts
    suction = -pw;
    break;
  }
  case 5:{//partially saturated medium =Van Genuchten model
    suction = -pw;
    break;
  }
  case 6:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    suction = -pw;
    break;
  }
  case 7:{//isotropic consolidation according to Kuklik with changing porosity from cam-clay model
    suction = -pw;
    break;
  }
  case 8:{//isotropic consolidation according to Kuklik with changing porosity from MEFEL cam-clay model
    suction = -pw;
    break;
  }
  default:{
    fprintf (stderr,"\n unknown model type is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  } 
  return(suction);
}
